/*
  Copyright 2014 Melin Software HB
  
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
  
      http://www.apache.org/licenses/LICENSE-2.0
  
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.net.*;


public class ListenMeOS {

  final static int MeOSPort = 21338;

  public enum RunnerStatus {
    StatusOK(1),
    StatusDNS(20), 
    StatusMP(3), 
    StatusDNF(4), 
    StatusDQ(5), 
    StatusMAX(6), 
    StatusUnknown(0), 
    StatusNotCompetiting(99);
    
    private int value;

    RunnerStatus(int value) {
      this.value = value;
    }
    
    public static RunnerStatus fromValue(int value) {
      for (RunnerStatus st : RunnerStatus.values()) {     
        if (st.value == value)
          return st;
      }
      throw new IllegalArgumentException();     
    }
  }
  
  static class SocketPunchInfo {
    int runnerId;
    int controlId;
    int status;
    int time;
    
    void deserialize(byte[] data) throws IOException {
      ByteArrayInputStream is = new ByteArrayInputStream(data);
      DataInputStream in = new DataInputStream(is);
      in.readInt(); // Competition id
      runnerId = Integer.reverseBytes(in.readInt());
      controlId = Integer.reverseBytes(in.readInt());
      status = Integer.reverseBytes(in.readInt());
      time = Integer.reverseBytes(in.readInt());
    }
    
    @Override
    public String toString() {
      String res = "Runner: " + runnerId + "\n";
      res += "Control: " + controlId + "\n";
      res += "Status: " + RunnerStatus.fromValue(status) + "\n";
      res += "Time: " + (time / 3600) + ":" + ((time/60) % 60) + ":"  + (time % 60) + "\n";
      return res;
    }
  }


  public static void main(String args[]) throws Exception {
    DatagramSocket serverSocket = new DatagramSocket(MeOSPort);
    try {
      byte[] receiveData = new byte[1024];
      while(true) {
        DatagramPacket receivePacket = new DatagramPacket(receiveData, receiveData.length);
        serverSocket.receive(receivePacket);
        byte[] data = receivePacket.getData();
        
        SocketPunchInfo pi = new SocketPunchInfo();
        pi.deserialize(data);       
        System.out.println("RECEIVED: " + pi);
      }
    }
    finally {
      serverSocket.close();
    }
  }
}
